@@ -28,8 +28,11 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.FileOutputStream;
 import java.io.IOException;
+import java.io.InputStream;
 import java.io.OutputStream;
+import java.io.OutputStreamWriter;
 import java.io.Reader;
+import java.io.Writer;
 import java.net.URL;
 import java.util.Properties;
 
@@ -38,7 +41,7 @@ import java.util.Properties;
  *
  * @author Peter Donald
  * @author Matthew Hawthorne
- * @version $Id: FileUtilsTest.java 3786 2006-08-31 19:31:06Z evenisse $
+ * @version $Id: FileUtilsTest.java 7069 2007-11-14 21:48:34Z olamy $
  * @see FileUtils
  */
 public final class FileUtilsTest
@@ -858,4 +861,217 @@ public final class FileUtilsTest
         compareFile.delete();
     }
 
+    /**
+     * TEST FAILS ON WINDOWS.
+     *
+     * Failing to specify an encoding means the default platform encoding, which
+     * varies from machine to machine.  There is no guarantee that what this
+     * creates is usable.
+     *
+     * This test should be deleted.
+     *
+     * @throws IOException
+     */
+    public void disabledTestFileRead() throws IOException
+    {
+        File testFile = new File( getTestDirectory(), "testFileRead.txt" );
+        String testFileName = testFile.getAbsolutePath();
+        // unicode escaped Japanese hiragana, "aiueo" + Umlaut a
+        String testString = "\u3042\u3044\u3046\u3048\u304a\u00e4";
+        Writer writer = null;
+        try
+        {
+            writer = new OutputStreamWriter( new FileOutputStream( testFile ) );
+            writer.write( testString );
+            writer.flush();
+        }
+        finally
+        {
+            writer.close();
+        }
+        assertEquals( "testString should be equal", testString, FileUtils.fileRead( testFile ) );
+        assertEquals( "testString should be equal", testString, FileUtils.fileRead( testFileName ) );
+        testFile.delete();
+    }
+
+    public void testFileReadWithEncoding() throws IOException
+    {
+        String encoding = "UTF-8";
+        File testFile = new File( getTestDirectory(), "testFileRead.txt" );
+        String testFileName = testFile.getAbsolutePath();
+        // unicode escaped Japanese hiragana, "aiueo" + Umlaut a
+        String testString = "\u3042\u3044\u3046\u3048\u304a\u00e4";
+        Writer writer = null;
+        try
+        {
+            writer = new OutputStreamWriter( new FileOutputStream( testFile ), encoding );
+            writer.write( testString );
+            writer.flush();
+        }
+        finally
+        {
+            writer.close();
+        }
+        assertEquals( "testString should be equal", testString, FileUtils.fileRead( testFile, "UTF-8" ) );
+        assertEquals( "testString should be equal", testString, FileUtils.fileRead( testFileName, "UTF-8" ) );
+        testFile.delete();
+    }
+
+    public void testFileAppend() throws IOException
+    {
+        String baseString = "abc";
+        File testFile = new File( getTestDirectory(), "testFileAppend.txt" );
+        String testFileName = testFile.getAbsolutePath();
+        Writer writer = null;
+        try
+        {
+            writer = new OutputStreamWriter( new FileOutputStream( testFile ) );
+            writer.write( baseString );
+            writer.flush();
+        }
+        finally
+        {
+            writer.close();
+        }
+        // unicode escaped Japanese hiragana, "aiueo" + Umlaut a
+        String testString = "\u3042\u3044\u3046\u3048\u304a\u00e4";
+        FileUtils.fileAppend( testFileName, testString );
+        assertEqualContent( ( baseString + testString ).getBytes(), testFile );
+        testFile.delete();
+    }
+
+    public void testFileAppendWithEncoding() throws IOException
+    {
+        String baseString = "abc";
+        String encoding = "UTF-8";
+        File testFile = new File( getTestDirectory(), "testFileAppend.txt" );
+        String testFileName = testFile.getAbsolutePath();
+        Writer writer = null;
+        try
+        {
+            writer = new OutputStreamWriter( new FileOutputStream( testFile ), encoding );
+            writer.write( baseString );
+            writer.flush();
+        }
+        finally
+        {
+            writer.close();
+        }
+        // unicode escaped Japanese hiragana, "aiueo" + Umlaut a
+        String testString = "\u3042\u3044\u3046\u3048\u304a\u00e4";
+        FileUtils.fileAppend( testFileName, encoding, testString );
+        assertEqualContent( ( baseString + testString ).getBytes( encoding ), testFile );
+        testFile.delete();
+    }
+
+    public void testFileWrite() throws IOException
+    {
+        File testFile = new File( getTestDirectory(), "testFileWrite.txt" );
+        String testFileName = testFile.getAbsolutePath();
+        // unicode escaped Japanese hiragana, "aiueo" + Umlaut a
+        String testString = "\u3042\u3044\u3046\u3048\u304a\u00e4";
+        FileUtils.fileWrite( testFileName, testString );
+        assertEqualContent( testString.getBytes(), testFile );
+        testFile.delete();
+    }
+
+    public void testFileWriteWithEncoding() throws IOException
+    {
+        String encoding = "UTF-8";
+        File testFile = new File( getTestDirectory(), "testFileWrite.txt" );
+        String testFileName = testFile.getAbsolutePath();
+        // unicode escaped Japanese hiragana, "aiueo" + Umlaut a
+        String testString = "\u3042\u3044\u3046\u3048\u304a\u00e4";
+        FileUtils.fileWrite( testFileName, encoding, testString );
+        assertEqualContent( testString.getBytes( encoding ), testFile );
+        testFile.delete();
+    }
+
+    /**
+     * Workaround for the following Sun bugs. They are fixed in JDK 6u1 and JDK 5u11.
+     *
+     * @see <a href="http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4403166">Sun bug id=4403166</a>
+     * @see <a href="http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6182812">Sun bug id=6182812</a>
+     * @see <a href="http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6481955">Sun bug id=6481955</a>
+     *
+     * @throws Exception
+     */
+    public void testDeleteLongPathOnWindows()
+        throws Exception
+    {
+        if ( System.getProperty( "os.name" ).toLowerCase().indexOf( "windows" ) == -1 )
+        {
+            return;
+        }
+
+        File a = new File( getTestDirectory(), "longpath" );
+        a.mkdir();
+        File a1 = new File( a, "a" );
+        a1.mkdir();
+
+        StringBuffer path = new StringBuffer( "" );
+        for ( int i = 0; i < 100; i++ )
+        {
+            path.append( "../a/" );
+        }
+
+        File f = new File( a1, path.toString() + "test.txt" );
+
+        InputStream is = new StringInputStream( "Blabla" );
+        OutputStream os = new FileOutputStream( f.getCanonicalFile() );
+        IOUtil.copy( is, os );
+        IOUtil.close( is );
+        IOUtil.close( os );
+
+        FileUtils.forceDelete( f );
+
+        File f1 = new File( a1, "test.txt" );
+        if ( f1.exists() )
+        {
+            throw new Exception( "Unable to delete the file :" + f1.getAbsolutePath() );
+        }
+    }
+    
+    //Test for bug PLXUTILS-10
+    public void testCopyFileOnSameFile()
+        throws IOException
+    {
+        String content = "ggrgreeeeeeeeeeeeeeeeeeeeeeeoierjgioejrgiojregioejrgufcdxivbsdibgfizgerfyaezgv!zeez";
+        final File theFile = File.createTempFile( "test", ".txt" );
+        FileUtils.fileAppend( theFile.getAbsolutePath(), content );
+
+        assertTrue( theFile.length() > 0 );
+        // Now copy file over itself
+        FileUtils.copyFile( theFile, theFile );
+
+        // This should not fail
+        assertTrue( theFile.length() > 0 );
+    }    
+
+    public void testExtensions()
+        throws Exception
+    {
+
+        String[][] values = {
+            { "fry.frozen", "frozen" },
+            { "fry", "" },
+            { "fry.", "" },
+            { "/turanga/leela/meets.fry", "fry" },
+            { "/3000/turanga.leela.fry/zoidberg.helps", "helps" },
+            { "/3000/turanga.leela.fry/zoidberg.", "" },
+            { "/3000/turanga.leela.fry/zoidberg", "" },
+            { "/3000/leela.fry.bender/", "" },
+            { "/3000/leela.fry.bdner/.", "" },
+            { "/3000/leela.fry.bdner/foo.bar.txt", "txt" } };
+
+        for ( int i = 0; i < values.length; i++ )
+        {
+            String fileName = values[i][0].replace( '/', File.separatorChar );
+            String ext = values[i][1];
+            String computed = FileUtils.extension( fileName );
+            assertEquals( "case [" + i + "]:" + fileName + " -> " + ext + ", computed : " + computed, ext, computed );
+        }
+
+    }
+
 }
